from requests import post, get, RequestException
from json import JSONDecodeError, loads
from whaller_client.exceptions import MethodError, ApiError, HttpError, InvalidResponseError

class ApiClient:
    def __init__(self, base_url: str):
        """
        Client to interact with the Whaller API.
        """
        self.api_base_url = base_url.rstrip('/') + '/api/'

    def call_json(self, endpoint: str, method: str, data: dict = {}, headers: dict = {}) -> dict:
        """
        Sends a request to the API and returns the response in JSON format.

        :param endpoint: Relative URL of the endpoint
        :param method: HTTP method ('GET' or 'POST')
        :param data: Data sent in the request
        :param headers: HTTP headers
        :return: API response as a dictionary
        :raises MethodError: If the HTTP method is invalid
        :raises HttpError: If the request fails
        :raises InvalidResponseError: If the JSON response is malformed
        :raises ApiError: If the API returns an error code
        """
        req_headers = {'Content-Type': 'application/json'}
        req_headers.update(headers)
        api_url = f'{self.api_base_url}{endpoint}'

        try:
            if method == 'POST':
                response = post(api_url, json=data, headers=req_headers)
            elif method == 'GET':
                response = get(api_url, params=data, headers=req_headers)
            else:
                raise MethodError(f"Invalid HTTP method: {method}")

            # Check if the HTTP status is an error (4xx, 5xx)
            response.raise_for_status()

            try:
                response_data = loads(response.content)
            except JSONDecodeError as e:
                raise InvalidResponseError(f"Invalid JSON response from {api_url}") from e

            # Check if the response contains an API error
            if 'error' in response_data:
                error_message = response_data['error'].get('message', 'Unknown error')
                raise ApiError(f"API Error: {error_message}")

            return response_data.get('result', {})

        except RequestException as e:
            raise HttpError(f"HTTP error on {api_url}: {str(e)}") from e



from base64 import b64encode
from whaller_client.api import ApiClient

class Authenticator:
    def __init__(self, client_id: str, client_token: str):
        self.client_id = client_id
        self.client_token = client_token
        self.token = None
        self.login = None
        self.password = None

    def set_credentials(self, login: str, password: str):
        self.login = login
        self.password = password

    def authenticate(self, api_client: ApiClient):
        token = (self.client_id + ':::' + self.client_token).encode("utf-8")
        header_application = {"X-Application": b64encode(token)}
        data = {'signin-login': self.login, 'signin-password': self.password}

        response = api_client.call_json('person/login', 'POST', data, header_application)
        self.token = response['auth_token']

    def get_bearer_token(self):
        if self.token is None:
            self.authenticate()
        return {"Authorization": "Bearer " + self.token}



class MethodError(Exception):
    """ Exception raised for an invalid HTTP method. """
    def __init__(self, message):
        self.message = message
    def __str__(self):
        return repr(self.message)

class ApiError(Exception):
    """ Exception raised when the API returns an error message. """
    def __init__(self, message):
        self.message = message
    def __str__(self):
        return repr(self.message)

class HttpError(Exception):
    """ Exception raised for an HTTP request issue (e.g., timeout, 404, 500, etc.). """
    def __init__(self, message):
        self.message = message
    def __str__(self):
        return repr(self.message)

class InvalidResponseError(Exception):
    """ Exception raised if the JSON response is malformed. """
    def __init__(self, message):
        self.message = message
    def __str__(self):
        return repr(self.message)

class UploadError(Exception):
    """ Exception raised for an error during file upload. """
    def __init__(self, message):
        self.message = message
    def __str__(self):
        return repr(self.message)



import os
import logging

class Logger():
    def __init__(self, name: str, logger_path: str = None, level: int = logging.ERROR):
        self.current_logger = self.init(name, level, logger_path)

    def init(self, filename, level, logger_path = None):
        if logger_path is None:
            logger_path = os.path.abspath(os.getcwd()) + '/logs/'

        if os.path.exists(logger_path) == False:
            os.makedirs(logger_path, exist_ok=True)

        logger = logging.getLogger(filename)
        logger.setLevel(level)
        if logger.hasHandlers() == False:
            filepath = logger_path + filename + '.log'
            fh = logging.FileHandler(filepath)
            fh.setLevel(level)
            formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
            fh.setFormatter(formatter)
            logger.addHandler(fh)
        return logger

    def debug(self, debug):
        if self.current_logger is None:
            return

        self.current_logger.debug(debug)

    def info(self, info):
        if self.current_logger is None:
            return

        self.current_logger.info(info)

    def warning(self, warning):
        if self.current_logger is None:
            return

        self.current_logger.warning(warning)

    def error(self, error):
        if self.current_logger is None:
            return

        self.current_logger.error(error)



import logging
from whaller_client.auth import Authenticator
from whaller_client.api import ApiClient
from whaller_client.logger import Logger

class Client:
    def __init__(self, base_url:str, client_id:str, client_token:str) -> None:
        self.authenticator = Authenticator(client_id, client_token)
        self.api = ApiClient(base_url)
        self.logger = Logger('api', level=logging.INFO)

    def set_credentials(self, login:str, password:str) -> None:
        self.authenticator.set_credentials(login, password)

    def authenticate(self):
        self.authenticator.authenticate(self.api)
    
    def get_api_token(self):
        return self.authenticator.get_bearer_token()

    def call_post(self, endpoint:str, data:dict={}, with_auth:bool=False) -> dict:
        headers = {}
        if with_auth:
            headers.update(self.get_api_token())
        return self.api.call_json(endpoint, "POST", data, headers)
    
    def call_get(self, endpoint:str, data:dict={}, with_auth:bool=False) -> dict:
        headers = {}
        if with_auth:
            headers.update(self.get_api_token())
        return self.api.call_json(endpoint, "GET", data, headers)



from setuptools import setup, find_packages

setup(
    name="whaller-client",
    version="0.1.0",
    packages=find_packages(),
    install_requires=[
        "requests"
    ],
    author="Whaller",
    author_email="contact@whaller.com",
    description="Python SDK for interacting with the Whaller API",
    url="https://github.com/whallerfr/client-python",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)
